@@ -105,7 +105,8 @@ class infoRegistry(metaclass=SingletonMeta):
     # ---------- main API ----------
 
     def get_video_info(self, url: str | None = None, video_id: str | None = None,
-                       force_refresh: bool = False, video_path: str | None = None) -> dict | None:
+                       force_refresh: bool = False, video_path: str | None = None,
+                       download=False) -> dict | None:
         # prune each call
         self.prune_registry(dry_run=False)
 
@@ -171,18 +172,17 @@ class infoRegistry(metaclass=SingletonMeta):
 # ---------------- Downloader ---------------- #
 
 class VideoDownloader:
-    def __init__(self, url, download_directory=None, user_agent=None,
+    def __init__(self, url=None,download_directory=None, user_agent=None,
                  video_extention="mp4", download_video=True,
                  video_path=None,
                  output_filename=None, ydl_opts=None,
                  registry=None, force_refresh=False,
-                 flat_layout: bool = False):
+                 flat_layout: bool = False,video_url =None):
 
-        self.url = get_corrected_url(url=url)
+        self.url = get_corrected_url(url or video_url) 
         self.video_urls = self.url if isinstance(self.url, list) else [self.url]
 
-        self.registry = registry or infoRegistry(video_root=download_directory,
-                                                 flat_layout=flat_layout)
+        self.registry = registry or infoRegistry(video_root=download_directory,flat_layout=flat_layout)
         self.ydl_opts = ydl_opts or {}
         self.get_download = download_video
         self.user_agent = user_agent
@@ -252,38 +252,68 @@ class VideoDownloader:
                 self._download_single(url)
         finally:
             self.stop()
+    
+    def _download_single(self, url=None,download_directory=None, user_agent=None,
+                         video_extention="mp4", download_video=True,
+                         video_path=None, output_filename=None, ydl_opts=None,
+                         registry=None, force_refresh=False,
+                         flat_layout: bool = False,video_url =None):
+         url = get_corrected_url(url or video_url or self.url) 
 
-    def _download_single(self, video_url: str,video_path:str=None):
-        logger.info(f"[VideoDownloader] Processing: {video_url}")
-
-        if "youtube.com/watch" in video_url and "v=" not in video_url:
+         ydl_opts = ydl_opts or self.ydl_opts or {}
+         get_download = get_download or self.get_download
+         user_agent = user_agent or self.user_agent
+         
+         force_refresh = force_refresh or self.force_refresh
+         
+         
+         download_directory = download_directory or self.download_directory
+         video_extention = video_extention or self.video_extention
+         output_filename = output_filename or self.output_filename
+         video_path = video_path or self.video_path
+         if video_path:
+             dirname = os.path.dirname(video_path)
+             output_filename = dirname if dirname else download_directory or get_video_root(download_directory)
+             basename = os.path.basename(video_path)
+             filename,ext = os.path.splitext(filename)
+             output_filename = filename if filename else output_filename or 'video'
+             video_extention = ext if ext else video_extention or '.mp4'
+             
+         
+         output_filename = output_filename or 'video'
+         download_directory = download_directory  or get_video_root(download_directory)
+         flat_layout = flat_layout or self.flat_layout
+         registry = registry or self.registry or infoRegistry(
+             video_root=download_directory,
+             flat_layout=flat_layout
+             )
+         logger.info(f"[VideoDownloader] Processing: {video_url}")
+         self.url = get_corrected_url(url or video_url or self.url) 
+         if "youtube.com/watch" in video_url and "v=" not in video_url:
             logger.debug(f"[VideoDownloader] Skipping bare watch URL: {video_url}")
             return None
 
-        info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
-        if info and info.get("video_path") and os.path.isfile(info["video_path"]):
+         info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
+         if info and info.get("video_path") and os.path.isfile(info["video_path"]):
             logger.info(f"[VideoDownloader] Already cached: {info['video_path']}")
             return info
 
-        try:
-            outtmpl = os.path.join(self.download_directory, "%(id)s.%(ext)s")
+         try:
+            video_extention = video_extention or "mp4"
+            outtmpl = os.path.join(self.download_directory, "%(id)s.%(video_extention)s")
             with yt_dlp.YoutubeDL(self._build_ydl_opts(outtmpl)) as ydl:
                 raw_info = ydl.extract_info(video_url, download=self.get_download)
 
             video_id = raw_info.get("id") or generate_video_id(raw_info.get("title") or "video")
 
-            # 🔑 flat or nested
-            dirbase = self.download_directory if self.flat_layout else os.path.join(self.download_directory, video_id)
-            os.makedirs(dirbase, exist_ok=True)
-
             temp_path = ydl.prepare_filename(raw_info)
             _, ext = os.path.splitext(temp_path)
-            ext = (ext.lstrip(".") or raw_info.get("ext") or "mp4").lower()
+            ext = (ext.lstrip(".") or raw_info.get("ext") or video_extention).lower()
             if not re.match(r"^[a-z0-9]+$", ext):
                 logger.warning(f"[VideoDownloader] Invalid ext {ext}, forcing mp4")
-                ext = "mp4"
+                ext = video_extention or "mp4"
             directory =info.get('directory')
-            final_path = os.path.join(directory, f"video.{ext}")
+            final_path = os.path.join(directory, f"{output_filename}.{ext}")
             if temp_path != final_path and os.path.isfile(temp_path):
                 shutil.move(temp_path, final_path)
 
@@ -305,9 +335,62 @@ class VideoDownloader:
             return info
 
 
-        except Exception as e:
+         except Exception as e:
             logger.error(f"[VideoDownloader] Download failed: {e}")
             return None
+def get_videoDownloader(url=None,
+                   download_directory=None,
+                   user_agent=None,
+                   video_extention=None,
+                   download_video=True,
+                   video_path=None,
+                   output_filename=None,
+                   ydl_opts=None,
+                   registry=None,
+                   force_refresh=False,
+                   flat_layout: bool = False,
+                   video_url=None):
+    videoDownload_mgr = get_videoDownloader(
+        url=url,
+        download_directory=download_directory,
+        user_agent=user_agent,
+        video_extention=video_extention,
+        download_video=download_video,
+        video_path=video_path,
+        output_filename=output_filename,
+        ydl_opts=ydl_opts,
+        registry=registry,
+        flat_layout=flat_layout,
+        video_url=video_url
+        )
+    return videoDownload_mgr
+def download_video(url=None,
+                   download_directory=None,
+                   user_agent=None,
+                   video_extention=None,
+                   download_video=True,
+                   video_path=None,
+                   output_filename=None,
+                   ydl_opts=None,
+                   registry=None,
+                   force_refresh=False,
+                   flat_layout: bool = False,
+                   video_url=None):
+    
+        videoDownload_mgr = get_videoDownloader(
+            url=url,
+            download_directory=download_directory,
+            user_agent=user_agent,
+            video_extention=video_extention,
+            download_video=download_video,
+            video_path=video_path,
+            output_filename=output_filename,
+            ydl_opts=ydl_opts,
+            registry=registry,
+            flat_layout=flat_layout,
+            video_url=video_url
+            )
+        return videoDownload_mgr._download_single()
 def get_registryManager(
     video_directory=None,
     envPath=None,
@@ -327,7 +410,7 @@ def get_video_info(
     ):
     url = url or video_url
     registryMgr = get_registryManager(video_directory=video_directory, envPath=envPath, info_directory=info_directory)
-    return registryMgr.get_video_info(url=url, video_id=video_id, force_refresh=force_refresh,video_path=video_path)
+    return registryMgr.get_video_info(url=url, video_id=video_id, force_refresh=force_refresh,video_path=video_path,download=download)
 def get_video_info_spec(
     key=None,
     url=None,
